`timescale 1ns/100ps
/******************************************************************
 *  This is the Top level Finite State Machine (FSM) for the RAM
 *  Built In Self Test module
 *****************************************************************/
module RAMBIST_FSM(Clock, Resetn, rd_address, wr_address,
                   pattern_select, wr_en, rd_en, cmp_en);
// Make the address_pointer most significant bit generic to allow
// for use of this top level FSM for different depth RAM blocks
parameter ADDRESS_MSB = 5;
input Clock, Resetn;
input [ADDRESS_MSB:0] rd_address, wr_address;
output [1:0] pattern_select;
output wr_en, rd_en, cmp_en;

reg [1:0] pattern_select;
parameter //Define the different RAM pattern types
CHECKERS  = 0, // 00 pattern is 0101 then 1010
INCREMENT = 1, // 01 pattern is incrementing pattern
GRAY      = 2, // 10 pattern is gray code pattern
ALTERNATE = 3; // 11 pattern is 1111 then 0000

reg wr_en, rd_en, cmp_en;

reg [2:0] rambist_top_state /* synthesis syn_encoding = "sequential" */ ;
parameter 
// Define the allowable states for the rambist_top_state
 IDLE                    = 0,
 START_WRITE             = 1,
 /****************************************************************************
  * The READ is delayed to insure that the write data and read data are not
  * the same for the patterns which simply toggle (CHECKER and ALTERNATE).
  * It also insures that the data is stored in the RAM cell for at least
  * two clock cycles before reading.
  ***************************************************************************/
 DELAY_READ_START        = 2, 
 START_READ              = 3,
 WAIT_FINAL_READ_ADDRESS = 4,
 LAST_COMPARE            = 5,
 NEXT_PATTERN            = 6
;

always @(posedge Clock or negedge Resetn)
 if (Resetn==1'b0) begin
  rambist_top_state <= IDLE;  
  pattern_select <= CHECKERS;
  wr_en <= 0;
  rd_en <= 0;
  cmp_en <= 0;
 end
 else begin
  wr_en <= wr_en;
  rd_en <= rd_en;
  cmp_en <= cmp_en;
  case (rambist_top_state)
   IDLE:                    begin
                             rambist_top_state <= START_WRITE;
                             wr_en <= 1'b1;
                             rd_en <= 1'b0;
                             cmp_en <= 1'b0;
                            end
   START_WRITE:             begin
                             rambist_top_state <= DELAY_READ_START;
                             wr_en <= 1'b1;
                             rd_en <= 1'b0;
                             cmp_en <= 1'b0;
                            end
   DELAY_READ_START:        begin
                             rambist_top_state <= START_READ;
                             wr_en <= 1'b1;
                             rd_en <= 1'b1;
                             cmp_en <= 1'b0;
                            end
   START_READ:              begin
                             rambist_top_state <= WAIT_FINAL_READ_ADDRESS;
                             wr_en <= 1'b1;
                             rd_en <= 1'b1;
                             cmp_en <= 1'b1;
                            end
   WAIT_FINAL_READ_ADDRESS: begin 
                             if (&rd_address) begin
                              rambist_top_state <= LAST_COMPARE;
                              rd_en <= 1'b0;
                             end
                             else begin
                              rambist_top_state <= WAIT_FINAL_READ_ADDRESS;
                              rd_en <= 1'b1;
                             end
                             if (&wr_address) wr_en <= 0;
                             else if(wr_en) wr_en <= 1;
                             cmp_en <= 1;
                            end
   LAST_COMPARE:            begin
                             rambist_top_state <= NEXT_PATTERN;
                             wr_en <= 1'b0;
                             rd_en <= 1'b0;
                             cmp_en <= 1'b0;
                            end
   NEXT_PATTERN:            begin
                             rambist_top_state <= START_WRITE;
                             pattern_select <= pattern_select + 1;
                             wr_en <= 1'b0;
                             rd_en <= 1'b0;
                             cmp_en <= 1'b0;
                            end
   default:                 begin
                             rambist_top_state <= IDLE;  
                             pattern_select <= CHECKERS;
                             wr_en <= 0;
                             rd_en <= 0;
                             cmp_en <= 0;
                            end
  endcase
 end
endmodule
